/******************************************************************************
*
* project name:    Generic Start-Utility
* file name:       start.c
* initial date:    15/09/2000
* author:          Zeljko Juric (zjuric@utic.net.ba)
*                  thomas.nussbaumer@gmx.net (misc. fixes)
*									 gdietsche@email.com (size optimization and leak detection)
*									 
* description:     launcher for any NOSTUB program which overcomes the 8kB 
*                  limit on AMS 2.03 and the 24kB limit on AMS 2.04/2.05
*
* THANX TO ZELJKO JURIC WHO RELEASES THIS CODE AS PART OF HIS SCOTTFREE DRIVER
* AND GAVE ME PERMISSION TO RE-USE IT
*
* ++++++++++++++++ Revision History ++++++++++++++++
*
* 22/11/2000: v 2.00 released - using now enter_ghost_space() and EX_patch()
*	30/04/2001	v 3.00					-	Small Size Optimization; 
*																Minor rewording for size;
*																capability for leak detection added (if wanted)
*																can start prgms with a ( after the prgm's name
*															- Support for TIGCC .91 SP1
**********************************************************************************/

#define NO_EXIT_SUPPORT	//support for TIGCC .91 sp1 and greater
#include <tigcclib.h>

int _ti89, _ti92plus;         // Produce both .89z and .9xz files

//not official TIGCC stuff, but I (Greg) think that it is safe
#undef	HomePushEStack
#define HomePushEStack	_rom_call(void,(void),10F)

#define fatal(s) {ST_showHelp(s); return;}

//Define This to get a report on bytes (+)gained or (-)lost
//#define LeakWatch

//--------------------------------------------------------------------------
// Note: This program is dirty-written to be as small as possible, so trying
// to understand how it works may be a good brain training for C experts :-)
//--------------------------------------------------------------------------
void _main(void)
{  
    char      *fptr,*cptr;
    unsigned   plen;
    SYM_ENTRY *SymPtr;
    ESI        argptr;
    int        argtype;
    char       tmpstr[100];
    char      *fname;
    char 			*ptmpstr;
    
   //unused variables
    //unsigned offs, *eptr, *pptr;

   
   #ifdef LeakWatch
   	unsigned long freemem=HeapAvail();
   #endif
   
   
    InitArgPtr(argptr);
    argtype = GetArgType(argptr);

    if (argtype != STR_TAG) fatal("use: start(\"name\")");
    
   //This is intentional... the estack pointer will be changed to skip the 
   //first arg so the called prog does not get that argument
    fname = GetStrnArg(top_estack);	//Some one please tell me if this is bad!
    //fname = GetStrnArg(argptr);

 	//ignore anything after the last (
    if((ptmpstr=strchr(fname,'(')))
    	*ptmpstr=0;
       
    tmpstr[0] = 0;
    strcpy(tmpstr+1,fname);
		//sprintf(tmpstr+1,"%s",fname);
		
    SymPtr = DerefSym(SymFind(tmpstr+strlen(fname)+1));
    
    if (!SymPtr) {
        sprintf(tmpstr,"%s not found",fname);
        fatal(tmpstr);
    }
		
    plen = *(int*)(cptr=fptr=HLock(SymPtr->handle))+3;
  
    // forcing AMS 2_xx users to archive their program is not
    // necessary anymore (heaven-thanx for enter_ghost_space())

    //if (AMS_2xx && !SymPtr->flags.bits.archived) {
    //    HeapUnlock(SymPtr->handle);
    //    fatal("Please archive program");
    //}
  
    if (SymPtr->flags.bits.archived || SymPtr->flags.bits.locked) {
        if (!(cptr=malloc(plen))) {
            HeapUnlock(SymPtr->handle);
            fatal("Low Ram");
        }    
        memcpy(cptr,fptr,plen);    
    }

    // relocate "by hand" (EX_patch will do the same)
    
    //for (eptr=pptr=(unsigned*)(cptr+plen-4);*pptr;pptr-=2);
    //while (pptr<eptr) offs=*++pptr,*(char**)(cptr+2+*++pptr)=cptr+0x40002+offs;

    enter_ghost_space();
    EX_patch(cptr + 0x40002, cptr + plen - 2);
    ASM_call(cptr + 0x40002);
    
    if (cptr!=fptr) free(cptr);

    HeapUnlock(SymPtr->handle);
    
    #ifdef LeakWatch
    	if((freemem=HeapAvail()-freemem)!=0)
    	{
	    	push_longint(freemem);
  	  	HomePushEStack();
    	
				sprintf(tmpstr,"RAM Gained/Lost: %ld",freemem);	
				ST_helpMsg(tmpstr);
			}
    #endif
}
